@@ -23,16 +23,15 @@
 
 #include "tags.h"
 #include "keys.h"
-#include "docgenerator.h"
-#include "chartranslator.h"
-#include "textgen.h"
-#include "colormap.h"
+#include "textgenerator.h"
+#include "textstyles.h"
+#include "textstylebuilder.h"
 
 // global
 #include "maingeneratormap.h"
 
-GeneratorFactory::GeneratorFactory(const string &t) :
-  char_translator (0), title (t)
+GeneratorFactory::GeneratorFactory(TextStylesPtr styles, Tags *tgs) :
+  textStyles(styles), tags(tgs)
 {
 }
 
@@ -40,18 +39,6 @@ GeneratorFactory::~ GeneratorFactory()
 {
   if (generatormap)
     delete generatormap;
-
-  if (MapOfColors)
-    delete MapOfColors;
-}
-
-CharTranslator *
-GeneratorFactory::getCharTranslator()
-{
-  if (! char_translator)
-    char_translator = createCharTranslator();
-
-  return char_translator;
 }
 
 GeneratorMap *
@@ -63,11 +50,8 @@ GeneratorFactory::createGeneratorMap()
 void
 GeneratorFactory::createGenerators()
 {
-  MapOfColors = createColorMap ();
   generatormap = createGeneratorMap();
 
-  const Tags *tags = getTags();
-
   TextGenerator *text_gen;
   for (Tags::const_iterator it = tags->begin(); it != tags->end(); ++it)
     {
@@ -80,18 +64,19 @@ GeneratorFactory::createGenerators()
   generatormap->setDefaultGenerator(normal);
 }
 
-#define GET_DECORATED( d ) ( d ? d : final_dec )
-
 TextGenerator *
 GeneratorFactory::createDefaultGenerator()
 {
-  return createGenerator (NORMAL);
+  if (textStyles->onestyle.empty())
+    return createGenerator (NORMAL);
+  else
+    return new TextGenerator(textStyles->onestyle);
 }
 
 TextGenerator *
 GeneratorFactory::createGenerator( const string &key )
 {
-  const Tag *tag = getTag( key ) ;
+  const Tag *tag = tags->GetTag( key ) ;
 
   if ( ! tag ) // no options
     return new TextGenerator ;
@@ -102,32 +87,49 @@ GeneratorFactory::createGenerator( const string &key )
 TextGenerator *
 GeneratorFactory::createGenerator( const Tag *tag )
 {
-  resetTagBuffers();
+  if (! textStyles->onestyle.empty())
+    return new TextGenerator(textStyles->onestyle.subst_style(tag->GetName()));
+
+  TextStyleBuilder textStyleBuilder(textStyles->starting_template, textStyles->style_separator);
+
+  textStyleBuilder.start();
+
+  TagStylesPtr tagStyles = tag->getTagStyles();
+  if (tagStyles.get()) {
+    for (TagStylesIterator it = tagStyles->begin(); it != tagStyles->end(); ++it) {
+      switch( *it ){
+        case ISBOLD:
+          textStyleBuilder.add(textStyles->bold);
+          break;
+        case ISITALIC:
+          textStyleBuilder.add(textStyles->italics);
+          break;
+        case ISUNDERLINE:
+          textStyleBuilder.add(textStyles->underline);
+          break;
+        case ISFIXED:
+          textStyleBuilder.add(textStyles->fixed);
+          break;
+        case ISNOTFIXED:
+          textStyleBuilder.add(textStyles->notfixed);
+          break;
+      }
+    }
+  }
 
   const string &color = tag->GetColor() ;
   if ( color.size () )
-    {
-     string c;
-     if ( color[0] == '#' && color.size () == 7 )
-       c = color; // FIXME: handle direct colors not only for HTML
-     else
-       c = MapOfColors->getColor (color);
+  {
+    string c;
+    if ( color[0] == '#' && color.size () == 7 )
+      c = color; // FIXME: handle direct colors not only for HTML
+    else
+      c = textStyles->colorMap->getColor (color);
 
-     addColor( c ) ;
-    }
+    textStyleBuilder.add(textStyles->color.subst_style(c));
+  }
 
-  if ( tag->IsBold() )
-    addBold();
-  if ( tag->IsItalic() )
-    addItalic();
-  if ( tag->IsUnderline() )
-    addUnderline();
+  TextStyle style = textStyleBuilder.end();
 
-  return new TextGenerator(getPreTag(), getPostTag());
-}
-
-DocGenerator *
-GeneratorFactory::createDocGenerator()
-{
-  return new DocGenerator (title);
+  return new TextGenerator(style);
 }
