@@ -18,6 +18,10 @@
 
 using namespace std;
 
+RegExpStateBuilder builder;
+// moved here so that the nested lists of states and formatters are still alive
+// (i.e., the shared_ptr are still reachable)
+
 LangDefLoader::LangDefLoader()
 {
 }
@@ -31,7 +35,6 @@ RegExpStatePtr
 LangDefLoader::get_lang_def(const string &path, const string &file)
 {
   LangElems *elems = parse_lang_def(path.c_str(), file.c_str());
-  RegExpStateBuilder builder; // FIXME do not create a builder every time!
   RegExpStatePtr state = builder.build(elems);
   delete elems;
 
