@@ -29,10 +29,10 @@ using namespace std;
 static const string buildex(const string &s);
 static const string buildex_pre(const string &s);
 
-void add_exp(RegExpStatePtr state, const string &orig, const string &exp, RegExpFormatterPtr f)
+void add_exp(RegExpStatePtr state, const string &orig, const string &exp, ParserInfo *parserInfo, RegExpFormatterPtr f)
 {
   try {
-    state->add_exp(exp, f);
+    state->add_exp(exp, parserInfo, f);
   } catch (boost::bad_expression &e) {
     printError("wrong original expression: " + orig);
     throw e;
@@ -150,7 +150,7 @@ RegExpStateBuilder::build(StringListLangElem *elem, RegExpStatePointer state)
 
   RegExpFormatterPtr formatter(new RegExpFormatter(name));
 
-  add_exp(state, exp_string, buildex_pre(exp_string), formatter);
+  add_exp(state, exp_string, buildex_pre(exp_string), elem, formatter);
   build(static_cast<StateStartLangElem *>(elem), state);
 }
 
@@ -195,23 +195,27 @@ RegExpStateBuilder::build(DelimitedLangElem *elem, RegExpStatePointer state)
     }
   } else {
     inner = RegExpStatePtr(new RegExpState); // for internal elements
+    nested_states.push_back(inner);
     inner->current_formatter = 1;
     inner->add_normal_formatter(RegExpFormatterPtr(new RegExpFormatter(NORMAL)));
 
     RegExpFormatterPtr exit(new RegExpFormatter(name, RegExpStatePtr(), true, elem->exitAll()));
     if (end)
-      add_exp(inner, end_string, buildex_pre(end_string), exit);
+      add_exp(inner, end_string, buildex_pre(end_string), elem, exit);
     else
-      inner->add_exp(buildex("\\z"), exit);
+      inner->add_exp(buildex("\\z"), elem, exit);
 
     if (escape) {
       add_exp(inner, escape_string, buildex_pre(escape_string + "."),
+              elem,
               RegExpFormatterPtr(new RegExpFormatter(name)));
     }
 
     if (elem->isNested()) {
       RegExpFormatterPtr nested(new RegExpFormatter(name, inner));
-      add_exp(inner, start_string, buildex_pre(start_string), nested);
+      nested_formatters.push_back(nested);
+      RegExpStatePtr s = nested->getNextState();
+      add_exp(inner, start_string, buildex_pre(start_string), elem, nested);
     }
   }
 
@@ -220,7 +224,7 @@ RegExpStateBuilder::build(DelimitedLangElem *elem, RegExpStatePointer state)
   }
 
   RegExpFormatterPtr formatter(new RegExpFormatter(name, inner));
-  add_exp(state, exp_string, buildex_pre(exp_string), formatter);
+  add_exp(state, exp_string, buildex_pre(exp_string), elem, formatter);
   build(static_cast<StateStartLangElem *>(elem), state);
 }
 
