@@ -1,15 +1,16 @@
 #ifndef TAGS_H
 #define TAGS_H
 
-#define SPACE_CHAR " "
-
-#define ISBOLD 0x1
-#define ISITALIC 0x2
-#define ISUNDERLINE 0x4
-
 #include "my_set.h"
 
 #include <string>
+#include <list>
+#include <boost/shared_ptr.hpp>
+
+typedef enum { ISBOLD=1, ISITALIC, ISUNDERLINE, ISFIXED, ISNOTFIXED } TagStyle;
+typedef std::list<TagStyle> TagStyles;
+typedef boost::shared_ptr<TagStyles> TagStylesPtr;
+typedef TagStyles::const_iterator TagStylesIterator;
 
 using std::string;
 
@@ -18,22 +19,18 @@ class Tag
  protected:
   string TagName ;
   string color ;
-  int flags ;
+  TagStylesPtr tagStyles;
 
  public:
-  Tag( const string &n ) : TagName( n ), flags( 0 ) {}
+  Tag( const string &n ) : TagName( n ) {}
   Tag( const string &n, const string &c );
 
-  void SetFlags( int f ) { flags = f ; }
+  void setStyles( TagStylesPtr t ) { tagStyles = t; }
+  const TagStylesPtr getTagStyles() const { return tagStyles; }
 
   string GetName() const { return TagName ; }
   string GetColor() const { return color ; }
   void SetColor( const string &col ) { color = col ; }
-
-  bool IsBold() const { return ( flags & ISBOLD ) ; }
-  bool IsItalic() const { return ( flags & ISITALIC ) ; }
-  bool IsUnderline() const { return ( flags & ISUNDERLINE ) ; }
-
   void Print() ;
 } ;
 
@@ -58,6 +55,7 @@ class Tags : protected set<Tag *, TagLess>
   using container_type::const_iterator;
 
   Tags() {}
+  ~Tags();
   bool AddTag( Tag *t ) { return insert( t ).second ;  }
   Tag *GetTag( const string &name ) ;
 } ;
@@ -74,10 +72,4 @@ Tags::GetTag( const string &name )
   return 0;
 }
 
-void setTags( Tags *t ) ;
-Tag *getTag( const string &name ) ;
-const Tags *getTags();
-void createDefaultTags() ;
-Tag *newDefaultTag( const string &tag, const string &color ) ;
-
 #endif
