@@ -1,7 +1,7 @@
 //
 // C++ Interface: textformatter
 //
-// Description: 
+// Description:
 //
 //
 // Author: Lorenzo Bettini <bettini@gnu.org>, (C) 2004
@@ -13,31 +13,37 @@
 #define TEXTFORMATTER_H
 
 #include <string>
+#include <boost/shared_ptr.hpp>
 
 /**
 format text, possibly preprocessing text before generating output
 
 @author Lorenzo Bettini
 */
+
+class TextFormatter;
+
+typedef boost::shared_ptr<TextFormatter> TextFormatterPtr;
+
 class TextFormatter
 {
-  TextFormatter *decorator;
-  
-  const std::string _preformat(const std::string &text); 
-  
+  TextFormatterPtr decorator;
+
+  const std::string _preformat(const std::string &text);
+
 public:
-  TextFormatter(TextFormatter *f = 0);
+  TextFormatter(TextFormatterPtr f = TextFormatterPtr());
 
   virtual ~TextFormatter();
-  
-  void setFormatter(TextFormatter *f);
+
+  void setFormatter(TextFormatterPtr f);
 
   void format(const std::string &elem, const std::string &text);
-  void format_nl(const std::string &elem, const std::string &text = "\n");
-    
+  void format_nl(const std::string &text = "\n");
+
 protected:
- 
-  virtual const std::string preformat(const std::string &text); 
+
+  virtual const std::string preformat(const std::string &text);
 
 };
 
