@@ -17,8 +17,6 @@ RegExpEngine::~RegExpEngine()
 }
 
 #include <fstream>
-#include <string>
-#include <stack>
 #include <iostream>
 #include <stdlib.h>
 
@@ -34,8 +32,6 @@ RegExpEngine::~RegExpEngine()
 
 using namespace std;
 
-typedef stack<RegExpStatePtr> stack_of_states;
-
 typedef enum { FOUND_EOF=0, FOUND_NL, FOUND_END } load_line_ret;
 
 load_line_ret
@@ -80,14 +76,14 @@ RegExpEngine::process_file(const char *file)
   std::string::const_iterator start, end;
   boost::match_results<std::string::const_iterator> what;
   boost::match_flag_type flags;
-  stack_of_states states_stack;
 
-  RegExpStatePtr initial_state = currentstate;
+  initial_state = currentstate;
 
   load_line_ret ret;
   while ((ret = load_line(s, *is)) != FOUND_EOF)
     {
       bool matched = true;
+      bool found_eol = false;
       start = s.begin();
       end = s.end();
       flags = boost::match_default;
@@ -97,24 +93,26 @@ RegExpEngine::process_file(const char *file)
           {
             string prefix = what.prefix();
             if (prefix.size())
-              formatter->format(currentstate->get_elem(), prefix);
+              format(-1, prefix);
 
             for (unsigned int i = 1; i < what.size(); ++i) {
               if (what[i].matched) {
-                formatter->format(currentstate->get_elem(i), string(what[i].first, what[i].second));
+                format(i, string(what[i].first, what[i].second));
                 if (currentstate->formatters[i]->getNextState()) {
-                  states_stack.push(currentstate);
-                  currentstate = currentstate->formatters[i]->getNextState();
+                  enterState(i);
                 } else if (currentstate->formatters[i]->exit_state) {
                   if (currentstate->formatters[i]->exit_all) {
-                    currentstate = initial_state;
-                    states_stack = stack_of_states();
+                    exitAll();
                   } else {
-                    currentstate = states_stack.top();
-                    states_stack.pop();
+                    exitState();
                   }
                 }
                 start = what[i].second;
+                if (!(*start)) {
+                  if (found_eol)
+                    matched = false; // we had already matched end of line
+                  found_eol = true;
+                }
                 break; // no other match is possible
               }
             }
@@ -123,13 +121,13 @@ RegExpEngine::process_file(const char *file)
           }
         else
           {
-            formatter->format(currentstate->get_elem(), string(start, end));
+            format(-1, string(start, end));
             matched = false;
           }
       }
 
       if (ret == FOUND_NL)
-        formatter->format_nl(currentstate->get_elem(), "\n");
+        formatter->format_nl("\n");
     }
 
   if (file)
@@ -138,19 +136,30 @@ RegExpEngine::process_file(const char *file)
   currentstate = initial_state; // reset the initial state
 }
 
-int process_file(const char *file, TextFormatter *pre, const string &path, const string &lang_file)
+void
+RegExpEngine::enterState(int index)
 {
-  RegExpStatePtr initial_state = LangDefLoader::get_lang_def(path, lang_file);
-
-  try{
-    printMessage("Processing " + string((file ? file : "standard input")) + " with regex");
-    printMessage("Using language definition " + lang_file);
-    RegExpEngine engine(initial_state, pre);
-    engine.process_file(file);
-  }
-  catch(...)
-  {
-    exitError("error during regex processing");
-  }
-  return 0;
+  states_stack.push(currentstate);
+  currentstate = currentstate->formatters[index]->getNextState();
 }
+
+void
+RegExpEngine::exitState()
+{
+  currentstate = states_stack.top();
+  states_stack.pop();
+}
+
+void
+RegExpEngine::exitAll()
+{
+  currentstate = initial_state;
+  states_stack = stack_of_states();
+}
+
+void
+RegExpEngine::format(int index, const std::string &s)
+{
+  formatter->format(currentstate->get_elem(index), s);
+}
+
