@@ -25,12 +25,16 @@ RegExpFormatter::RegExpFormatter(const string &el, RegExpStatePtr r, bool exit,
 
 void RegExpFormatter::setNextState(RegExpStatePtr r)
 {
-  next_state = r;
+  next_state_strong = r;
 }
 
 RegExpStatePtr RegExpFormatter::getNextState() const
 {
-  return next_state;
+  RegExpStatePtr next = next_state.lock();
+  if (!next)
+    return next_state_strong;
+
+  return next;
 }
 
 const string &
@@ -40,7 +44,7 @@ RegExpState::get_elem(int index)
 }
 
 void
-RegExpState::add_exp(const std::string &s, RegExpFormatterPtr f)
+RegExpState::add_exp(const std::string &s, ParserInfo *parserInfo, RegExpFormatterPtr f)
 {
   const string &ex = buffer.str();
   if (ex.size())
@@ -49,6 +53,7 @@ RegExpState::add_exp(const std::string &s, RegExpFormatterPtr f)
   buffer << s;
 
   formatters.push_back(f);
+  subExpressions.push_back(make_pair(s, *parserInfo));
 }
 
 void
