@@ -23,5 +23,7 @@ int
 main( int argc, char * argv[] )
 {
   StartApp start_app;
-  return start_app.start (argc, argv);
+  int result = start_app.start (argc, argv);
+
+  return result;
 }
