@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <iostream>
 #include <fstream>
+#include <assert.h>
 #include <string.h>
 
 #include "colors.h"
@@ -38,17 +39,17 @@
 #include "reportbugs.h"
 #include "parsetags.h"
 
-#include "htmlgeneratorfactory.h"
-#include "xhtmlgeneratorfactory.h"
-#include "cssgeneratorfactory.h"
-#include "escgeneratorfactory.h"
+#include "generatorfactory.h"
 #include "srcuntabifier.h"
 #include "chartranslator.h"
 #include "langdefloader.h"
 #include "lineoutputgenerator.h"
 #include "langmap.h"
 #include "regexpengine.h"
+#include "regexpenginedebug.h"
 #include "docgenerator.h"
+#include "textstyles.h"
+#include "outlangdefparserfun.h"
 
 // for globals
 #include "linenumdigit.h"
@@ -74,7 +75,7 @@ gengetopt_args_info args_info ;     // command line structure
 static void print_cgi_header();
 
 StartApp::StartApp() :
-  docgenerator(0), formatter(0), langmap(0), generator_factory(0),
+    docgenerator(0), formatter(0), langmap(new LangMap), outlangmap(new LangMap), generator_factory(0),
   entire_doc (0), verbose (0), cssUrl (0),
   use_css (0), is_cgi (0), gen_version(true),
   generate_line_num(false), generate_ref(false)
@@ -86,15 +87,14 @@ StartApp::~StartApp()
   // cout << "destroying StartApp..." << endl;
   cmdline_parser_free(&args_info);
 
+  // TODO: fix all the pointer dependences
+
   if (formatter)
     delete formatter;
 
   if (docgenerator)
     delete docgenerator;
 
-  if (langmap)
-    delete langmap;
-
   if (generator_factory)
     delete generator_factory;
 }
@@ -138,26 +138,8 @@ StartApp::start(int argc, char * argv[])
       return EXIT_SUCCESS;
     }
 
-  bool format_html = (strcmp (args_info.out_format_arg, "html") == 0);
-  bool format_xhtml = (strcmp (args_info.out_format_arg, "xhtml") == 0);
-  bool format_esc = (strcmp (args_info.out_format_arg, "esc") == 0);
-
   gen_version = (args_info.gen_version_flag != 0);
 
-  const char *ext = 0;
-
-  if (! format_html && ! format_xhtml && ! format_esc)
-    {
-      cerr << PACKAGE << ": output format " << args_info.out_format_arg
-           << " not recognized" << endl;
-      return EXIT_FAILURE;
-    }
-
-  if (format_html || format_xhtml)
-    ext = ".html";
-  else
-    ext = ".txt";
-
   /* initialization of global symbols */
   inputFileName = outputFileName = 0 ;
   sout = 0 ;
@@ -169,7 +151,7 @@ StartApp::start(int argc, char * argv[])
   header_fileName = args_info.header_arg ;
   footer_fileName = args_info.footer_arg ;
   verbose = args_info.verbose_given ;
-  const string tags_file = args_info.tags_file_arg;
+  const string style_file = args_info.style_file_arg;
 
   if ( args_info.tab_given > 0 )
     tabSpaces = args_info.tab_arg ;
@@ -185,8 +167,7 @@ StartApp::start(int argc, char * argv[])
 
   entire_doc =
     (! args_info.no_doc_given) &&
-    ( args_info.doc_given || (docTitle != 0) || use_css ||
-      docHeader || docFooter ) ;
+    ( args_info.doc_given || (docTitle != 0) || use_css ) ;
 
   string inputFileName;
   if (args_info.input_given)
@@ -219,82 +200,100 @@ StartApp::start(int argc, char * argv[])
     return (EXIT_FAILURE);
   }
 
+  if (args_info.check_outlang_given) {
+    cout << "checking " << args_info.check_outlang_arg << "... ";
+    textstyles = parse_outlang_def(data_dir.c_str(), args_info.check_outlang_arg);
+    cout << "OK" << endl;
+    return (EXIT_SUCCESS);
+  }
+
   string lang_map = args_info.lang_map_arg;
-  langmap = new LangMap(data_dir, lang_map);
+  assert(lang_map.size());
+  if (! args_info.lang_def_given)
+    langmap = LangMapPtr(new LangMap(data_dir, lang_map));
+
+  string outlang_map = args_info.outlang_map_arg;
+  assert(outlang_map.size());
+  if (! args_info.outlang_def_given)
+    outlangmap = LangMapPtr(new LangMap(data_dir, outlang_map));
 
   if (args_info.lang_list_given) {
-    cout << "Supported languages (file extensions) and associated language definition files\n\n";
+    cout << "Supported languages (file extensions)\nand associated language definition files\n\n";
     langmap->print();
     return (EXIT_SUCCESS);
   }
 
-  parseTags(data_dir, tags_file) ;
+  if (args_info.outlang_list_given) {
+    cout << "Supported output languages\nand associated language definition files\n\n";
+    outlangmap->print();
+    return (EXIT_SUCCESS);
+  }
+
+  Tags *tags = parseTags(data_dir, style_file);
 
   outputbuffer = new OutputBuffer;
 
-  if (format_esc)
-    {
-      if (use_css)
-        {
-          cerr << PACKAGE << ": cannot use css with esc output format" << endl;
-          return (EXIT_FAILURE);
-        }
+  string title;
+  string doc_header;
+  string doc_footer;
+  string css_url;
 
-      if (args_info.line_number_ref_given)
-        {
-          cerr << PACKAGE << ": cannot generate line references in esc output format" << endl;
-          return (EXIT_FAILURE);
-        }
+  if (docTitle)
+    title = docTitle;
+  if ((! docTitle) && inputFileName.size())
+    title = inputFileName;
+  if (docHeader)
+    doc_header = docHeader;
+  if (docFooter)
+    doc_footer = docFooter;
+  if (cssUrl)
+    css_url = cssUrl;
 
-      generator_factory = new EscGeneratorFactory;
+  if (args_info.line_number_ref_given)
+    args_info.line_number_given = args_info.line_number_ref_given;
+
+  string outlangfile;
+
+  if (! args_info.outlang_def_given) {
+    string out_format = args_info.out_format_arg;
+
+    if (use_css)
+      out_format += "-css";
+
+    if (entire_doc)
+      out_format += "-doc";
+
+    outlangfile = outlangmap->get_file(out_format);
+
+    if (! outlangfile.size()) {
+      cerr << PACKAGE << ": ";
+      cerr << "output language " << out_format
+          << " not handled" << endl;
+      return EXIT_FAILURE ;
     }
-  else
-    {
-     string title;
-     string doc_header;
-     string doc_footer;
-     string css_url;
+  } else {
+    outlangfile = args_info.outlang_def_arg;
+  }
 
-     if (docTitle)
-       title = docTitle;
-     if ((! docTitle) && inputFileName.size())
-        title = inputFileName;
-     if (docHeader)
-       doc_header = docHeader;
-     if (docFooter)
-       doc_footer = docFooter;
-     if (cssUrl)
-       css_url = cssUrl;
+  textstyles = parse_outlang_def(data_dir.c_str(), outlangfile.c_str());
 
-     if (args_info.line_number_ref_given)
-       args_info.line_number_given = args_info.line_number_ref_given;
+  if (! textstyles->file_extension.size() && ! outputFileName.size()) {
+    cerr << PACKAGE << ": ";
+    cerr << "empty file extension in output language file " <<
+         outlangfile << endl;
+    return EXIT_FAILURE ;
+  }
 
-     if( use_css )
-       {
-         generator_factory =
-           new CssGeneratorFactory (title,
-                                    inputFileName, doc_header,
-                                    doc_footer, css_url);
-       }
-     else
-       {
-         if (format_xhtml && ! args_info.css_given)
-           generator_factory =
-             new XHtmlGeneratorFactory
-              (title,
-               inputFileName, doc_header,
-               doc_footer, css_url);
-         else
-           generator_factory =
-            new HtmlGeneratorFactory
-             (title,
-              inputFileName, doc_header,
-              doc_footer, css_url);
-       }
-     }
+  const string ext = "." + textstyles->file_extension;
+
+  generator_factory = new GeneratorFactory(textstyles, tags);
 
   generator_factory->createGenerators ();
-  docgenerator = generator_factory->createDocGenerator();
+  docgenerator = new DocGenerator(title, inputFileName,
+                                  doc_header, doc_footer,
+                                  css_url, entire_doc,
+                                  textstyles->docTemplate.toStringBegin(),
+                                  textstyles->docTemplate.toStringEnd());;
 
   if ( is_cgi )
     print_cgi_header() ;
@@ -312,8 +311,7 @@ StartApp::start(int argc, char * argv[])
   else
     formatter = new TextFormatter();
 
-  CharTranslator *chartranslator = generator_factory->getCharTranslator();
-
+  TextFormatterPtr chartranslator(textstyles->charTranslator);
   formatter->setFormatter(chartranslator);
 
   if (args_info.lang_def_arg)
@@ -347,6 +345,12 @@ StartApp::start(int argc, char * argv[])
   delete outputbuffer;
   outputbuffer = 0;
 
+  if (textstyles)
+    delete textstyles;
+
+  if (tags)
+    delete tags;
+
   return (result);
 }
 
@@ -374,6 +378,27 @@ StartApp::print_version()
   cout << "GNU " << PACKAGE << " " << VERSION << endl;
 }
 
+int process_file(const char *file, TextFormatter *pre, const string &path, const string &lang_file)
+{
+  RegExpStatePtr initial_state = LangDefLoader::get_lang_def(path, lang_file);
+
+  try{
+    printMessage("Processing " + string((file ? file : "standard input")) + " with regex");
+    printMessage("Using language definition " + lang_file);
+    RegExpEnginePtr engine;
+    if (args_info.debug_langdef_given)
+      engine = RegExpEnginePtr(new RegExpEngineDebug(initial_state, pre));
+    else
+      engine = RegExpEnginePtr(new RegExpEngine(initial_state, pre));
+    engine->process_file(file);
+  }
+  catch(...)
+  {
+    exitError("error during regex processing");
+  }
+  return 0;
+}
+
 int
 StartApp::processFile(const string &inputFileName, const string &outputFileName)
 {
@@ -419,14 +444,14 @@ StartApp::processFile(const string &inputFileName, const string &outputFileName)
   }
 
   if (generate_line_num)
-    outputgenerator = new LineOutputGenerator(*outputbuffer, *sout, generate_ref);
+    outputgenerator =
+      new LineOutputGenerator(*outputbuffer, *sout,
+        &(textstyles->anchor), generate_ref,
+        (args_info.line_number_ref_given ? args_info.line_number_ref_arg : ""));
   else
     outputgenerator = new OutputGenerator(*outputbuffer, *sout);
 
-  if ( entire_doc ) {
-    docgenerator->set_gen_version (gen_version);
-    docgenerator->generate_top ();
-  }
+  docgenerator->set_gen_version (gen_version);
 
   printMessage( "translating source code... ", cerr ) ;
 
@@ -504,10 +529,6 @@ StartApp::processFile(const string &inputFileName, const string &outputFileName)
     docgenerator->generate_end_doc ();
 
     printMessage( "done !", cerr ) ;
-
-    if ( entire_doc ) {
-      docgenerator->generate_bottom ();
-    }
   }
   else // we're in failsafe mode so we simply copy the file to the output
   {
